;	QLAYER.LSP - For use with QLAYER.DCL
;	(Quick Layer on/off freeze/thaw program)

(defun c:qlayer ( / index do_it cmdval selset)
	;*** save the cmdecho variable how we find it
	(setq cmdval (getvar "cmdecho"))
	
	;*** set lisp so no commands are echoed
	(setvar "cmdecho" 0)
	(setq layerlist (list))
	(setq index 0)
	(setq do_it 0)
	
	;*** ask the user to pick some objects
	(princ "\nPick objects on layers to process, " )
	(setq selset (ssget)) 
	
	;*** if something is selected then find the layers
	;	of everthing picked and make a list of them
	(if (/= selset nil) 
		(progn
			(repeat (sslength selset)	  
			 ;*** get the entities layer name
				(setq layer (cdr (assoc 8 (entget (ssname selset index)))))
			
				;*** if already in the list don't add it again
				(if (= (member layer layerlist) nil)
					(setq layerlist (append layerlist (list layer)))
				)
				(setq index (1+ index))
			)
		)
		
		;*** otherwise select all layers
		(setq layerlist (list "*"))
	)
	
	;*** load the layer dialog to query the user
	(layer_dialog)
	
	;*** if user selects OK then process layers
	(if (= do_it 1) 
		(progn
			(setq index 0)
		 
			;*** start layer command
			(command "layer")
			
			;*** check if THAW button is active or not
			(if (equal thaw "1") 
				;*** if active process layers to thaw them
				(layer_command "thaw")
				;*** otherwise freeze them
				(layer_command "freeze")
			)

			;*** check if ON button is active or not
			(if (equal on "1") 
				;*** if active process layers to turn them on
				(layer_command "on") 
				;*** otherwise turn them off
				(layer_command "off")
			)

			;*** end the layer command
			(command "") 
		)
	)
	;*** reset the command echo to how we found it
	(setvar "cmdecho" cmdval)
	(princ)
)

(defun layer_command (action / index)
	(setq index 0)
	;*** step through the list of picked layer names
	;	Layer command is still being run so only need
	;	to give the option keywords eg ON, OFF, FREEZE
	(repeat (length layerlist)
		(command action (nth index layerlist))
		(if (equal action "off")	  
			(progn
				(if (equal "*" (nth index layerlist)) (command "Y"))
				(if (equal (getvar "CLAYER") (nth index layerlist)) (command "Y"))
			)
		)
		(setq index (1+ index))
	)	
)

(defun qlayer_ok ( / index)
	;*** get values from dialog
	(setq on (get_tile "on"))
	(setq thaw (get_tile "thaw"))
	(setq lock (get_tile "lock"))

	;*** set flag that user pressed ok
	(setq do_it 1)
	;*** finish the dialog box bit
	(done_dialog) 
)

(defun Layer_dialog()
	;*** load DCL file
	(setq di_dcl_id (load_dialog "qlayer.dcl"))
	(if (< di_dcl_id 0) (exit))
	;*** display dialog
	(if (not (new_dialog "qlayer" di_dcl_id "" dim_pos)) (exit))

	;*** define the action to take when the user presses OK
	(action_tile "accept" "(qlayer_ok)")
	(start_dialog)				;returns after OK or CANCEL selected
	(unload_dialog di_dcl_id)	 ;free DCL from memory
)

;define error handling function to deal with cancelled commands
(defun *error* (msg / )
	(princ "error: ")
	(princ msg)
	(terpri)
	(shutdown)
)
